==> rn-shiki-0.0.15.tgz: package/scripts/build.js <==
import fs from 'node:fs'
import path from 'node:path'
import { fileURLToPath } from 'node:url'
import esbuild from 'esbuild'

const __dirname = path.dirname(fileURLToPath(import.meta.url))
const VENDOR_DIR = path.join(__dirname, '../vendor/shiki')

function createDeclarationFile(outputPath, name) {
  // Only create declarations for themes and langs
  if (!outputPath.includes('/themes/') && !outputPath.includes('/langs/')) {
    return
  }

  // Remove .js extension and create paths
  const jsPath = outputPath.replace(/\\/g, '/')
  const dtsPath = jsPath.replace('.js', '.d.ts')

  // Calculate the relative module path without stripping off 'vendor/shiki'
  const modulePath = `vendor/shiki/${path.relative(VENDOR_DIR, jsPath).replace('.js', '')}`

  // Determine correct type annotation
  const typeAnnotation = outputPath.includes('/themes/') ? 'ThemeInput' : 'LanguageInput'

  // Replace hyphens in name to create a valid TypeScript identifier
  const safeName = name.replace(/-/g, '_')

  const dtsContent = `\
declare module '${modulePath}' {
  const ${safeName}: import('shiki').${typeAnnotation};
  export default ${safeName};
}
`
  fs.writeFileSync(dtsPath, dtsContent)
}

// Build function
async function buildModule(inputPath, outputPath, typeName = 'any') {
  try {
    const result = await esbuild.build({
      entryPoints: [inputPath],
      bundle: true,
      write: false,
      format: 'cjs',
      platform: 'node',
      external: ['react', 'react-native'],
      define: {
        'process.env.NODE_ENV': '"production"',
      },
    })

    if (inputPath.includes('/themes/')) {
      const themeContent = result.outputFiles[0].text
      const match = themeContent.match(/JSON\.parse\('(.+?)'\)/)
      if (match) {
        fs.writeFileSync(outputPath, `module.exports = ${match[1]}`)
      }
      else {
        throw new Error('Could not find theme data in bundle')
      }
    }
    else {
      fs.writeFileSync(outputPath, result.outputFiles[0].text)
    }

    // Create declaration file for TypeScript, replacing hyphens in name
    createDeclarationFile(outputPath, path.basename(outputPath, '.js').replace(/-/g, '_'), typeName)
    return true
  }
  catch (error) {
    console.error(`Failed to build ${inputPath}:`, error)
    return false
  }
}

// Create initial manifest with metadata
function createManifest() {
  const shikiPkg = JSON.parse(fs.readFileSync(path.join(__dirname, '../node_modules/shiki/package.json'), 'utf8'))

  return {
    version: shikiPkg.version,
    buildDate: new Date().toISOString(),
    core: [],
    langs: {},
    themes: {},
  }
}

const shikiDir = path.join(__dirname, '../node_modules/shiki/dist')
const langFiles = fs
  .readdirSync(path.join(shikiDir, 'langs'))
  .filter(file => file.endsWith('.mjs'))
  .map(file => ({
    input: `shiki/dist/langs/${file}`,
    output: `vendor/shiki/langs/${file.replace('.mjs', '.js')}`,
    name: file.replace('.mjs', ''),
  }))

const themeFiles = fs
  .readdirSync(path.join(shikiDir, 'themes'))
  .filter(file => file.endsWith('.mjs'))
  .map(file => ({
    input: `shiki/dist/themes/${file}`,
    output: `vendor/shiki/themes/${file.replace('.mjs', '.js')}`,
    name: file.replace('.mjs', ''),
  }))

const coreModules = [
  {
    input: 'shiki/dist/core.mjs',
    output: 'vendor/shiki/core/core.js',
  },
  {
    input: 'shiki/dist/engine-javascript.mjs',
    output: 'vendor/shiki/core/engine-javascript.js',
  },
  ...langFiles,
  ...themeFiles,
]

// Initialize manifest
const manifest = createManifest()

// Ensure vendor directory exists
if (!fs.existsSync(VENDOR_DIR)) {
  fs.mkdirSync(VENDOR_DIR, { recursive: true })
}

// Create necessary subdirectories
;['core', 'langs', 'themes'].forEach((dir) => {
  const fullPath = path.join(VENDOR_DIR, dir)
  if (!fs.existsSync(fullPath)) {
    fs.mkdirSync(fullPath, { recursive: true })
  }
})

// Build all modules and update manifest
for (const module of coreModules) {
  const inputPath = path.join(__dirname, '../node_modules', module.input)
  const outputPath = path.join(__dirname, '..', module.output)

  const dir = path.dirname(outputPath)
  if (!fs.existsSync(dir)) {
    fs.mkdirSync(dir, { recursive: true })
  }

  await buildModule(inputPath, outputPath, module.type || 'any')
  console.log(`Built ${path.basename(module.input)} -> ${path.basename(module.output)}`)

  // Update manifest
  if (module.input.includes('/langs/')) {
    manifest.langs[module.name] = {
      file: path.relative(VENDOR_DIR, outputPath),
      built: new Date().toISOString(),
    }
  }
  else if (module.input.includes('/themes/')) {
    manifest.themes[module.name] = {
      file: path.relative(VENDOR_DIR, outputPath),
      built: new Date().toISOString(),
    }
  }
  else {
    manifest.core.push({
      name: module.name,
      file: path.relative(VENDOR_DIR, outputPath),
      built: new Date().toISOString(),
    })
  }
}

// Write manifest
fs.writeFileSync(path.join(VENDOR_DIR, 'manifest.json'), JSON.stringify(manifest, null, 2))

==> rn-shiki-0.0.16.tgz: package/scripts/build.js <==
import fs from 'node:fs'
import path from 'node:path'
import { fileURLToPath } from 'node:url'
import esbuild from 'esbuild'

const __dirname = path.dirname(fileURLToPath(import.meta.url))
const VENDOR_DIR = path.join(__dirname, '../src/vendor/shiki')

function createDeclarationFile(outputPath, name) {
  // Handle core types differently
  if (outputPath.includes('/core/')) {
    const dtsPath = outputPath.replace('.js', '.d.ts')
    const modulePath = outputPath.split('src/')[1].replace('.js', '')

    if (outputPath.includes('core.js')) {
      fs.writeFileSync(
        dtsPath,
        `\
import type { LanguageInput, ThemeInput } from 'shiki'

declare module '${modulePath}' {
  export interface HighlighterCoreOptions {
    engine: any
    langs: LanguageInput[]
    themes: ThemeInput[]
  }

  export function createHighlighterCore(options: HighlighterCoreOptions): Promise<{
    codeToTokens: (code: string, options: { lang: string; theme: string }) => any
  }>
}
`,
      )
      return
    }

    if (outputPath.includes('engine-javascript.js')) {
      fs.writeFileSync(
        dtsPath,
        `\
declare module '${modulePath}' {
  export function createJavaScriptRegexEngine(): {
    grammar: any
    getRuleStack: (line: string) => any
    tokenizeLine: (line: string, stack: any) => any
  }
}
`,
      )
      return
    }
  }

  // Only create declarations for themes and langs
  if (!outputPath.includes('/themes/') && !outputPath.includes('/langs/')) {
    return
  }

  // Remove .js extension and create paths
  const jsPath = outputPath.replace(/\\/g, '/')
  const dtsPath = jsPath.replace('.js', '.d.ts')

  // Get the path starting from 'vendor/shiki'
  const modulePath = jsPath.split('src/')[1].replace('.js', '')

  // Determine correct type annotation
  const typeAnnotation = outputPath.includes('/themes/') ? 'ThemeInput' : 'LanguageInput'

  // List of reserved words
  const reservedWords = ['for', 'switch', 'case', 'break', 'default', 'else', 'var', 'const', 'let', 'function', 'class']

  // Always prefix with underscore if it's a reserved word
  const varName = reservedWords.includes(name) ? `_${name}` : name.replace(/-/g, '_')

  const dtsContent = `\
declare module '${modulePath}' {
  const ${varName}: import('shiki').${typeAnnotation};
  export default ${varName};
}
`
  fs.writeFileSync(dtsPath, dtsContent)
}

async function buildModule(inputPath, outputPath, name) {
  try {
    const result = await esbuild.build({
      entryPoints: [inputPath],
      bundle: true,
      write: false,
      format: 'cjs',
      platform: 'node',
      external: ['react', 'react-native'],
      define: {
        'process.env.NODE_ENV': '"production"',
      },
    })

    if (inputPath.includes('/themes/')) {
      const themeContent = result.outputFiles[0].text
      const match = themeContent.match(/JSON\.parse\('(.+?)'\)/)
      if (match) {
        fs.writeFileSync(outputPath, `module.exports = ${match[1]}`)
      }
      else {
        throw new Error('Could not find theme data in bundle')
      }
    }
    else {
      fs.writeFileSync(outputPath, result.outputFiles[0].text)
    }

    // Create declaration file for TypeScript
    createDeclarationFile(outputPath, name || path.basename(outputPath, '.js'))
    return true
  }
  catch (error) {
    console.error(`Failed to build ${inputPath}:`, error)
    return false
  }
}

function createManifest() {
  const shikiPkg = JSON.parse(fs.readFileSync(path.join(__dirname, '../node_modules/shiki/package.json'), 'utf8'))

  return {
    version: shikiPkg.version,
    buildDate: new Date().toISOString(),
    core: [],
    langs: {},
    themes: {},
  }
}

const shikiDir = path.join(__dirname, '../node_modules/shiki/dist')
const langFiles = fs
  .readdirSync(path.join(shikiDir, 'langs'))
  .filter(file => file.endsWith('.mjs'))
  .map(file => ({
    input: `shiki/dist/langs/${file}`,
    output: `src/vendor/shiki/langs/${file.replace('.mjs', '.js')}`,
    name: file.replace('.mjs', ''),
  }))

const themeFiles = fs
  .readdirSync(path.join(shikiDir, 'themes'))
  .filter(file => file.endsWith('.mjs'))
  .map(file => ({
    input: `shiki/dist/themes/${file}`,
    output: `src/vendor/shiki/themes/${file.replace('.mjs', '.js')}`,
    name: file.replace('.mjs', ''),
  }))

const coreModules = [
  {
    input: 'shiki/dist/core.mjs',
    output: 'src/vendor/shiki/core/core.js',
  },
  {
    input: 'shiki/dist/engine-javascript.mjs',
    output: 'src/vendor/shiki/core/engine-javascript.js',
  },
  ...langFiles,
  ...themeFiles,
]

// Initialize manifest
const manifest = createManifest()

// Ensure vendor directory exists
if (!fs.existsSync(VENDOR_DIR)) {
  fs.mkdirSync(VENDOR_DIR, { recursive: true })
}

// Create necessary subdirectories
;['core', 'langs', 'themes'].forEach((dir) => {
  const fullPath = path.join(VENDOR_DIR, dir)
  if (!fs.existsSync(fullPath)) {
    fs.mkdirSync(fullPath, { recursive: true })
  }
})

// Build all modules and update manifest
for (const module of coreModules) {
  const inputPath = path.join(__dirname, '../node_modules', module.input)
  const outputPath = path.join(__dirname, '..', module.output)

  const dir = path.dirname(outputPath)
  if (!fs.existsSync(dir)) {
    fs.mkdirSync(dir, { recursive: true })
  }

  await buildModule(inputPath, outputPath, module.name)
  console.log(`Built ${path.basename(module.input)} -> ${path.basename(module.output)}`)

  // Update manifest
  if (module.input.includes('/langs/')) {
    manifest.langs[module.name] = {
      file: path.relative(VENDOR_DIR, outputPath),
      built: new Date().toISOString(),
    }
  }
  else if (module.input.includes('/themes/')) {
    manifest.themes[module.name] = {
      file: path.relative(VENDOR_DIR, outputPath),
      built: new Date().toISOString(),
    }
  }
  else {
    manifest.core.push({
      name: module.name || path.basename(module.output, '.js'),
      file: path.relative(VENDOR_DIR, outputPath),
      built: new Date().toISOString(),
    })
  }
}

// Write manifest
fs.writeFileSync(path.join(VENDOR_DIR, 'manifest.json'), JSON.stringify(manifest, null, 2))
